{% extends "admin/base_site.html" %}
{% load i18n %}

{% block userlinks %}
    <a href="{% url 'user-key-list' %}">{% translate 'Multi Factor Authentication' %}</a> /
    {{ block.super }}
{% endblock %}