# Generated by Django 3.2.4 on 2021-06-06 18:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_rest_mfa', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='userkey',
            name='key_type',
            field=models.CharField(choices=[('FIDO2', 'Fido2'), ('TOTP', 'Totp'), ('Trusted Device', 'Device'), ('Backup Codes', 'Backup Codes')], max_length=25),
        ),
    ]
